@@ -3,6 +3,7 @@
  *
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
+ * Copyright (C) 2004 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,30 +28,65 @@
 #include "misc/khtmllayout.h"
 #include "rendering/render_object.h"
 
+#include <loader.h>
+
 #include <qregion.h>
+#include <qmap.h>
+#include <qpixmap.h>
+
+namespace khtml {
+    class CachedImage;
+    class CachedObjectClient;
+}
 
 namespace DOM {
 
 class DOMString;
 
+class HTMLFormElementImpl;
+    
+class HTMLImageLoader: public khtml::CachedObjectClient {
+public:
+    HTMLImageLoader(ElementImpl* elt);
+    virtual ~HTMLImageLoader();
+
+    void updateFromElement();
+
+    void dispatchLoadEvent();
+
+    ElementImpl* element() const { return m_element; }
+    bool imageComplete() const { return m_imageComplete; }
+    khtml::CachedImage* image() const { return m_image; }
+
+    // CachedObjectClient API
+    virtual void notifyFinished(khtml::CachedObject *finishedObj);
+
+private:
+    ElementImpl* m_element;
+    khtml::CachedImage* m_image;
+    bool m_firedLoad : 1;
+    bool m_imageComplete : 1;
+};
+
 class HTMLImageElementImpl
     : public HTMLElementImpl
 {
+    friend class HTMLFormElementImpl;
 public:
-    HTMLImageElementImpl(DocumentPtr *doc);
-
+    HTMLImageElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
     ~HTMLImageElementImpl();
 
     virtual Id id() const;
 
-    virtual void parseAttribute(AttributeImpl *);
+    virtual bool mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const;
+    virtual void parseHTMLAttribute(HTMLAttributeImpl *);
 
     virtual void attach();
     virtual khtml::RenderObject *createRenderer(RenderArena *, khtml::RenderStyle *);
     virtual void detach();
 
-    long width() const;
-    long height() const;
+    long width(bool ignorePendingStylesheets = false) const;
+    long height(bool ignorePendingStylesheets = false) const;
 
     bool isServerMap() const { return ( ismap && !usemap.length() );  }
     QImage currentImage() const;
@@ -58,12 +94,24 @@ public:
     DOMString altText() const;
 
     DOMString imageMap() const { return usemap; }
+    
+    virtual bool isURLAttribute(AttributeImpl *attr) const;
 
+#if APPLE_CHANGES
+    QString compositeOperator() const { return _compositeOperator; }
+    const QPixmap &pixmap() { return m_imageLoader.image()->pixmap(); }
+#endif
+    
 protected:
+    HTMLImageLoader m_imageLoader;
     DOMString usemap;
     bool ismap;
+    HTMLFormElementImpl *m_form;
     QString oldIdAttr;
     QString oldNameAttr;
+#if APPLE_CHANGES
+    QString _compositeOperator;
+#endif
 };
 
 
@@ -80,7 +128,7 @@ public:
 
     virtual Id id() const;
 
-    virtual void parseAttribute(AttributeImpl *attr);
+    virtual void parseHTMLAttribute(HTMLAttributeImpl *attr);
 
     bool isDefault() const { return shape==Default; }
 
@@ -95,8 +143,7 @@ protected:
     khtml::Length* m_coords;
     int m_coordsLen;
     int lastw, lasth;
-    Shape shape  : 3;
-    bool nohref  : 1;
+    Shape shape;
 };
 
 
@@ -113,13 +160,13 @@ public:
 
     virtual DOMString getName() const { return name; }
 
-    virtual void parseAttribute(AttributeImpl *attr);
+    virtual void parseHTMLAttribute(HTMLAttributeImpl *attr);
 
     bool mapMouseEvent(int x_, int y_, int width_, int height_,
                        khtml::RenderObject::NodeInfo& info);
 private:
 
-    QString name;
+    DOMString name;
 };
 
 
