@@ -32,12 +32,15 @@
 #include <qevent.h>
 
 class KHTMLPart;
+class QPoint;
+class QStringList;
 
 namespace DOM {
 
 class AbstractViewImpl;
 class DOMStringImpl;
 class NodeImpl;
+class ClipboardImpl;
 
 // ### support user-defined events
 
@@ -57,6 +60,23 @@ public:
         MOUSEOVER_EVENT,
         MOUSEMOVE_EVENT,
         MOUSEOUT_EVENT,
+        // IE copy/paste events
+        BEFORECUT_EVENT,
+        CUT_EVENT,
+        BEFORECOPY_EVENT,
+        COPY_EVENT,
+        BEFOREPASTE_EVENT,
+        PASTE_EVENT,
+        // IE drag and drop events
+        DRAGENTER_EVENT,
+        DRAGOVER_EVENT,
+        DRAGLEAVE_EVENT,
+        DROP_EVENT,
+        DRAGSTART_EVENT,
+        DRAG_EVENT,
+        DRAGEND_EVENT,
+	// IE selection events
+	SELECTSTART_EVENT,
         // Mutation events
         DOMSUBTREEMODIFIED_EVENT,
         DOMNODEINSERTED_EVENT,
@@ -79,6 +99,10 @@ public:
 	RESIZE_EVENT,
 	SCROLL_EVENT,
         CONTEXTMENU_EVENT,
+#if APPLE_CHANGES
+        SEARCH_EVENT,
+#endif
+        INPUT_EVENT,
         // Keyboard events
 	KEYDOWN_EVENT,
 	KEYUP_EVENT,
@@ -93,7 +117,11 @@ public:
 	KHTML_MOVE_EVENT,
 	KHTML_ORIGCLICK_MOUSEUP_EVENT,
 	// XMLHttpRequest events
-	KHTML_READYSTATECHANGE_EVENT
+	KHTML_READYSTATECHANGE_EVENT,
+        // extensions
+        MOUSEWHEEL_EVENT,
+        HORIZONTALMOUSEWHEEL_EVENT,
+        numEventIds
     };
 
     EventImpl();
@@ -120,11 +148,14 @@ public:
     virtual bool isMouseEvent() const;
     virtual bool isMutationEvent() const;
     virtual bool isKeyboardEvent() const;
+    virtual bool isDragEvent() const;   // a subset of mouse events
+    virtual bool isClipboardEvent() const;
+    virtual bool isWheelEvent() const;
 
     bool propagationStopped() const { return m_propagationStopped; }
     bool defaultPrevented() const { return m_defaultPrevented; }
 
-    static EventId typeToId(DOMString type);
+    static EventId typeToId(const DOMString &type);
     static DOMString idToType(EventId id);
 
     void setDefaultHandled() { m_defaultHandled = true; }
@@ -178,11 +209,62 @@ protected:
 
 };
 
+class UIEventWithKeyStateImpl : public UIEventImpl {
+public:
+    UIEventWithKeyStateImpl() : m_ctrlKey(false), m_altKey(false), m_shiftKey(false), m_metaKey(false) { }
+    UIEventWithKeyStateImpl(EventId eventID, bool canBubbleArg, bool cancelableArg, AbstractViewImpl *viewArg,
+        long detailArg, bool ctrlKeyArg, bool altKeyArg, bool shiftKeyArg, bool metaKeyArg)
+        : UIEventImpl(eventID, canBubbleArg, cancelableArg, viewArg, detailArg)
+        , m_ctrlKey(ctrlKeyArg), m_altKey(altKeyArg), m_shiftKey(shiftKeyArg), m_metaKey(metaKeyArg) { }
 
+    bool ctrlKey() const { return m_ctrlKey; }
+    bool shiftKey() const { return m_shiftKey; }
+    bool altKey() const { return m_altKey; }
+    bool metaKey() const { return m_metaKey; }
 
+protected: // expose these so init functions can set them
+    bool m_ctrlKey : 1;
+    bool m_altKey : 1;
+    bool m_shiftKey : 1;
+    bool m_metaKey : 1;
+};
 
-// Introduced in DOM Level 2: - internal
-class MouseEventImpl : public UIEventImpl {
+// Internal only: Helper class for what's common between mouse and wheel events.
+class MouseRelatedEventImpl : public UIEventWithKeyStateImpl {
+public:
+    MouseRelatedEventImpl();
+    MouseRelatedEventImpl(EventId _id,
+                          bool canBubbleArg,
+                          bool cancelableArg,
+                          AbstractViewImpl *viewArg,
+                          long detailArg,
+                          long screenXArg,
+                          long screenYArg,
+                          long clientXArg,
+                          long clientYArg,
+                          bool ctrlKeyArg,
+                          bool altKeyArg,
+                          bool shiftKeyArg,
+                          bool metaKeyArg);
+    long screenX() const { return m_screenX; }
+    long screenY() const { return m_screenY; }
+    long clientX() const { return m_clientX; }
+    long clientY() const { return m_clientY; }
+    long layerX() const { return m_layerX; }
+    long layerY() const { return m_layerY; }
+protected: // expose these so MouseEventImpl::initMouseEvent can set them
+    long m_screenX;
+    long m_screenY;
+    long m_clientX;
+    long m_clientY;
+    void computeLayerPos();
+private:
+    long m_layerX;
+    long m_layerY;
+};
+
+// Introduced in DOM Level 2
+class MouseEventImpl : public MouseRelatedEventImpl {
 public:
     MouseEventImpl();
     MouseEventImpl(EventId _id,
@@ -199,20 +281,12 @@ public:
 		   bool shiftKeyArg,
 		   bool metaKeyArg,
 		   unsigned short buttonArg,
-		   NodeImpl *relatedTargetArg);
+		   NodeImpl *relatedTargetArg,
+                   ClipboardImpl *clipboardArg=0);
     virtual ~MouseEventImpl();
-    long screenX() const { return m_screenX; }
-    long screenY() const { return m_screenY; }
-    long clientX() const { return m_clientX; }
-    long clientY() const { return m_clientY; }
-    long layerX() const { return m_layerX; }
-    long layerY() const { return m_layerY; }
-    bool ctrlKey() const { return m_ctrlKey; }
-    bool shiftKey() const { return m_shiftKey; }
-    bool altKey() const { return m_altKey; }
-    bool metaKey() const { return m_metaKey; }
     unsigned short button() const { return m_button; }
     NodeImpl *relatedTarget() const { return m_relatedTarget; }
+    ClipboardImpl *clipboard() const { return m_clipboard; }
     void initMouseEvent(const DOMString &typeArg,
 			bool canBubbleArg,
 			bool cancelableArg,
@@ -229,26 +303,16 @@ public:
 			unsigned short buttonArg,
 			const Node &relatedTargetArg);
     virtual bool isMouseEvent() const;
-protected:
-    long m_screenX;
-    long m_screenY;
-    long m_clientX;
-    long m_clientY;
-    long m_layerX;
-    long m_layerY;
-    bool m_ctrlKey;
-    bool m_altKey;
-    bool m_shiftKey;
-    bool m_metaKey;
+    virtual bool isDragEvent() const;
+private:
     unsigned short m_button;
     NodeImpl *m_relatedTarget;
- private:
-    void computeLayerPos();
+    ClipboardImpl *m_clipboard;
 };
 
 
 // Introduced in DOM Level 3
-class KeyboardEventImpl : public UIEventImpl {
+class KeyboardEventImpl : public UIEventWithKeyStateImpl {
 public:
     KeyboardEventImpl();
     KeyboardEventImpl(QKeyEvent *key, AbstractViewImpl *view);
@@ -259,8 +323,8 @@ public:
                 const DOMString &keyIdentifierArg,
                 unsigned long keyLocationArg,
                 bool ctrlKeyArg,
-                bool shiftKeyArg,
                 bool altKeyArg,
+                bool shiftKeyArg,
                 bool metaKeyArg,
                 bool altGraphKeyArg);
     virtual ~KeyboardEventImpl();
@@ -272,18 +336,14 @@ public:
                 const DOMString &keyIdentifierArg,
                 unsigned long keyLocationArg,
                 bool ctrlKeyArg,
-                bool shiftKeyArg,
                 bool altKeyArg,
+                bool shiftKeyArg,
                 bool metaKeyArg,
                 bool altGraphKeyArg);
     
     DOMString keyIdentifier() const { return m_keyIdentifier; }
     unsigned long keyLocation() const { return m_keyLocation; }
     
-    bool ctrlKey() const { return m_ctrlKey; }
-    bool shiftKey() const { return m_shiftKey; }
-    bool altKey() const { return m_altKey; }
-    bool metaKey() const { return m_metaKey; }
     bool altGraphKey() const { return m_altGraphKey; }
     
     QKeyEvent *qKeyEvent() const { return m_keyEvent; }
@@ -297,10 +357,6 @@ private:
     QKeyEvent *m_keyEvent;
     DOMStringImpl *m_keyIdentifier;
     unsigned long m_keyLocation;
-    bool m_ctrlKey : 1;
-    bool m_shiftKey : 1;
-    bool m_altKey : 1;
-    bool m_metaKey : 1;
     bool m_altGraphKey : 1;
 };
 
@@ -340,6 +396,36 @@ protected:
     unsigned short m_attrChange;
 };
 
+class ClipboardEventImpl : public EventImpl {
+public:
+    ClipboardEventImpl();
+    ClipboardEventImpl(EventId _id, bool canBubbleArg, bool cancelableArg, ClipboardImpl *clipboardArg);
+    ~ClipboardEventImpl();
+
+    ClipboardImpl *clipboard() const { return m_clipboard; }
+    virtual bool isClipboardEvent() const;
+protected:
+    ClipboardImpl *m_clipboard;
+};
+
+// extension: mouse wheel event
+class WheelEventImpl : public MouseRelatedEventImpl
+{
+public:
+    WheelEventImpl();
+    WheelEventImpl(bool horizontal, long wheelDelta, AbstractViewImpl *,
+                   long screenXArg, long screenYArg,
+                   long clientXArg, long clientYArg,
+                   bool ctrlKeyArg, bool altKeyArg, bool shiftKeyArg, bool metaKeyArg);
+    bool isHorizontal() const { return m_horizontal; }
+    long wheelDelta() const { return m_wheelDelta; }
+
+private:
+    virtual bool isWheelEvent() const;
+
+    bool m_horizontal;
+    long m_wheelDelta;
+};
 
 class RegisteredEventListener {
 public:
@@ -356,5 +442,34 @@ private:
     RegisteredEventListener & operator=( const RegisteredEventListener & );
 };
 
-}; //namespace
+// State available during IE's events for drag and drop and copy/paste
+class ClipboardImpl : public khtml::Shared<ClipboardImpl> {
+public:
+    ClipboardImpl() { }
+    virtual ~ClipboardImpl();
+    // Is this operation a drag-drop or a copy-paste?
+    virtual bool isForDragging() const = 0;
+
+    virtual DOMString dropEffect() const = 0;
+    virtual void setDropEffect(const DOMString &s) = 0;
+    virtual DOMString effectAllowed() const = 0;
+    virtual void setEffectAllowed(const DOMString &s) = 0;
+    
+    virtual void clearData(const DOMString &type) = 0;
+    virtual void clearAllData() = 0;
+    virtual DOMString getData(const DOMString &type, bool &success) const = 0;
+    virtual bool setData(const DOMString &type, const DOMString &data) = 0;
+    
+    // extensions beyond IE's API
+    virtual QStringList types() const = 0;
+    
+    virtual QPoint dragLocation() const = 0;
+    virtual QPixmap dragImage() const = 0;
+    virtual void setDragImage(const QPixmap &, const QPoint &) = 0;
+    virtual const Node dragImageElement() = 0;
+    virtual void setDragImageElement(const Node &, const QPoint &) = 0;
+};
+
+} // namespace
+
 #endif
