@@ -76,6 +76,8 @@ void CharacterDataImpl::setData( const DOMString &_data, int &exceptioncode )
     
     dispatchModifiedEvent(oldStr);
     if(oldStr) oldStr->deref();
+    
+    getDocument()->removeAllMarkers(this);
 }
 
 unsigned long CharacterDataImpl::length() const
@@ -130,6 +132,10 @@ void CharacterDataImpl::insertData( const unsigned long offset, const DOMString
     
     dispatchModifiedEvent(oldStr);
     oldStr->deref();
+    
+    // update the markers for spell checking and grammar checking
+    uint length = arg.length();
+    getDocument()->shiftMarkers(this, offset, length);
 }
 
 void CharacterDataImpl::deleteData( const unsigned long offset, const unsigned long count, int &exceptioncode )
@@ -148,6 +154,10 @@ void CharacterDataImpl::deleteData( const unsigned long offset, const unsigned l
     
     dispatchModifiedEvent(oldStr);
     oldStr->deref();
+
+    // update the markers for spell checking and grammar checking
+    getDocument()->removeAllMarkers(this, offset, count);
+    getDocument()->shiftMarkers(this, offset + count, -count);
 }
 
 void CharacterDataImpl::replaceData( const unsigned long offset, const unsigned long count, const DOMString &arg, int &exceptioncode )
@@ -173,6 +183,11 @@ void CharacterDataImpl::replaceData( const unsigned long offset, const unsigned
     
     dispatchModifiedEvent(oldStr);
     oldStr->deref();
+    
+    // update the markers for spell checking and grammar checking
+    int diff = arg.length() - count;
+    getDocument()->removeAllMarkers(this, offset, count);
+    getDocument()->shiftMarkers(this, offset + count, diff);
 }
 
 DOMString CharacterDataImpl::nodeValue() const
@@ -257,6 +272,13 @@ unsigned long CharacterDataImpl::caretMaxRenderedOffset() const
     return r ? r->caretMaxRenderedOffset() : length();
 }
 
+bool CharacterDataImpl::rendererIsNeeded(RenderStyle *style)
+{
+    if (!str || str->l == 0)
+        return false;
+    return NodeImpl::rendererIsNeeded(style);
+}
+
 #ifndef NDEBUG
 void CharacterDataImpl::dump(QTextStream *stream, QString ind) const
 {
@@ -389,43 +411,41 @@ NodeImpl *TextImpl::cloneNode(bool /*deep*/)
 
 bool TextImpl::rendererIsNeeded(RenderStyle *style)
 {
-    if (!CharacterDataImpl::rendererIsNeeded(style)) {
+    if (!CharacterDataImpl::rendererIsNeeded(style))
         return false;
-    }
+
     bool onlyWS = containsOnlyWhitespace();
-    if (!onlyWS) {
+    if (!onlyWS)
         return true;
-    }
 
     RenderObject *par = parentNode()->renderer();
     
-    if (par->isTable() || par->isTableRow() || par->isTableSection()) {
+    if (par->isTable() || par->isTableRow() || par->isTableSection() || par->isTableCol())
         return false;
-    }
     
-    if (style->whiteSpace() == PRE) {
+    if (style->whiteSpace() == PRE)
         return true;
-    }
     
+    RenderObject *prev = previousRenderer();
+    if (prev && prev->isBR()) // <span><br/> <br/></span>
+        return false;
+        
     if (par->isInline()) {
         // <span><div/> <div/></span>
-        RenderObject *prev = previousRenderer();
-        if (prev && prev->isRenderBlock()) {
+        if (prev && prev->isRenderBlock())
             return false;
-        }
     } else {
-        RenderObject *prev = previousRenderer();
-        if (par->isRenderBlock() && !par->childrenInline() && (!prev || !prev->isInline())) {
+        if (par->isRenderBlock() && !par->childrenInline() && (!prev || !prev->isInline()))
             return false;
-        }
         
         RenderObject *first = par->firstChild();
+        while (first && first->isFloatingOrPositioned())
+            first = first->nextSibling();
         RenderObject *next = nextRenderer();
-        if (!first || next == first) {
+        if (!first || next == first)
             // Whitespace at the start of a block just goes away.  Don't even
             // make a render object for this text.
             return false;
-        }
     }
     
     return true;
@@ -477,6 +497,29 @@ DOMString TextImpl::toString() const
     return nodeValue();
 }
 
+#ifndef NDEBUG
+void TextImpl::formatForDebugger(char *buffer, unsigned length) const
+{
+    DOMString result;
+    DOMString s;
+    
+    s = nodeName();
+    if (s.length() > 0) {
+        result += s;
+    }
+          
+    s = nodeValue();
+    if (s.length() > 0) {
+        if (result.length() > 0)
+            result += "; ";
+        result += "value=";
+        result += s;
+    }
+          
+    strncpy(buffer, result.string().latin1(), length - 1);
+}
+#endif
+
 // ---------------------------------------------------------------------------
 
 CDATASectionImpl::CDATASectionImpl(DocumentPtr *impl, const DOMString &_text) : TextImpl(impl,_text)
