@@ -24,17 +24,10 @@
  */
 
 #import "KWQApplication.h"
-#import "KWQLogging.h"
 
 // The NSScreen Cocoa calls in this file should never throw, so
 // there's no need to block exceptions.
 
-QPalette QApplication::palette(const QWidget *p)
-{
-    static QPalette pal;
-    return pal;
-}
-
 int QDesktopWidget::width()
 {
     return (int)[[NSScreen mainScreen] frame].size.width;
