@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (C) 2004 Apple Computer, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -117,6 +117,13 @@ void KWQArrayImpl::duplicate(const void *data, size_t newSize)
     memcpy(d->data, data, newSize * d->itemSize);
 }
 
+void KWQArrayImpl::detach()
+{
+    if (d->refCount > 1) {
+        duplicate(d->data, d->numItems);
+    }
+}
+
 bool KWQArrayImpl::fill(const void *item, int numItems)
 {
     if (numItems == -1) {
