@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (C) 2004 Apple Computer, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,29 +26,37 @@
 #ifndef KWQKHTMLPart_H
 #define KWQKHTMLPart_H
 
-#include "khtml_part.h"
+#import "khtml_part.h"
 
-#include "dom_nodeimpl.h"
-#include "html_formimpl.h"
-#include "html_tableimpl.h"
+#import "dom_nodeimpl.h"
+#import "dom2_range.h"
 
 #import "WebCoreKeyboardAccess.h"
 
-#include <CoreFoundation/CoreFoundation.h>
+#import <CoreFoundation/CoreFoundation.h>
 
-#include <JavaVM/jni.h>
-#include <JavaScriptCore/jni_jsobject.h>
-#include <JavaScriptCore/runtime.h>
+#import <JavaScriptCore/jni_jsobject.h>
+#import <JavaScriptCore/npruntime.h>
+#import <JavaScriptCore/NP_jsobject.h>
+#import <JavaScriptCore/runtime.h>
 
-#include "KWQDict.h"
-#include "KWQClipboard.h"
+#import "KWQDict.h"
+#import "KWQClipboard.h"
+#import "KWQScrollBar.h"
 
 class KHTMLPartPrivate;
-class KWQClipboard;
 class KWQWindowWidget;
 
+namespace DOM {
+    class DocumentFragmentImpl;
+    class HTMLTableCellElementImpl;
+}
+
 namespace khtml {
     class RenderObject;
+    class RenderStyle;
+    class VisiblePosition;
+    struct DashboardRegionValue;
 }
 
 namespace KJS {
@@ -67,6 +75,7 @@ typedef DOMElement ObjCDOMElement;
 @class NSArray;
 @class NSAttributedString;
 @class NSColor;
+@class NSDictionary;
 @class NSEvent;
 @class NSFileWrapper;
 @class NSFont;
@@ -87,6 +96,7 @@ class KWQPageState;
 class NSArray;
 class NSAttributedString;
 class NSColor;
+class NSDictionary;
 class NSEvent;
 class NSFileWrapper;
 class NSFont;
@@ -98,6 +108,8 @@ class NSView;
 class WebCoreBridge;
 class WebScriptObject;
 
+typedef int NSWritingDirection;
+
 #endif
 
 enum KWQSelectionDirection {
@@ -131,6 +143,8 @@ public:
     
     void scrollToAnchor(const KURL &);
     void jumpToSelection();
+    QString advanceToNextMisspelling(bool startBeforeSelection = false);
+    bool scrollOverflow(KWQScrollDirection direction, KWQScrollGranularity granularity);
     
     void setEncoding(const QString &encoding, bool userChosen);
     void addData(const char *bytes, int length);
@@ -139,7 +153,6 @@ public:
     void setStatusBarText(const QString &status);
 
     void urlSelected(const KURL &url, int button, int state, const KParts::URLArgs &args);
-    bool requestObject(khtml::RenderPart *frame, const QString &url, const QString &serviceType, const QStringList &args);
     KParts::ReadOnlyPart *createPart(const khtml::ChildFrame &child, const KURL &url, const QString &mimeType);
 
     void scheduleClose();
@@ -183,9 +196,16 @@ public:
     void runJavaScriptAlert(const QString &message);
     bool runJavaScriptConfirm(const QString &message);
     bool runJavaScriptPrompt(const QString &message, const QString &defaultValue, QString &result);
+    bool locationbarVisible();
+    bool menubarVisible();
+    bool personalbarVisible();
+    bool scrollbarsVisible();
+    bool statusbarVisible();
+    bool toolbarVisible();
+
     void KWQKHTMLPart::addMessageToConsole(const QString &message,  unsigned int lineNumber, const QString &sourceID);
     using KHTMLPart::xmlDocImpl;
-    khtml::RenderObject *renderer();
+    khtml::RenderObject *renderer() const;
     void forceLayout();
     void forceLayoutWithPageWidthRange(float minPageWidth, float maxPageWidth);
     void sendResizeEvent();
@@ -212,10 +232,17 @@ public:
 
     QRect selectionRect() const;
     NSRect visibleSelectionRect() const;
+    void centerSelectionInVisibleArea() const;
     NSImage *selectionImage() const;
     NSImage *snapshotDragImage(DOM::Node node, NSRect *imageRect, NSRect *elementRect) const;
 
-    NSFont *fontForCurrentPosition() const;
+    NSFont *fontForSelection(bool *hasMultipleFonts) const;
+    NSDictionary *fontAttributesForSelectionStart() const;
+    
+    NSWritingDirection baseWritingDirectionForSelectionStart() const;
+
+    void markMisspellingsInAdjacentWords(const khtml::VisiblePosition &);
+    void markMisspellings(const khtml::Selection &);
 
     NSFileWrapper *fileWrapperForElement(DOM::ElementImpl *);
     NSAttributedString *attributedString(DOM::NodeImpl *startNode, int startOffset, DOM::NodeImpl *endNode, int endOffset);
@@ -225,20 +252,32 @@ public:
     void mouseDown(NSEvent *);
     void mouseDragged(NSEvent *);
     void mouseUp(NSEvent *);
-    void sendFakeEventsAfterWidgetTracking(NSEvent *initiatingEvent);
     void mouseMoved(NSEvent *);
     bool keyEvent(NSEvent *);
-    bool lastEventIsMouseUp();
+    bool wheelEvent(NSEvent *);
 
+    void sendFakeEventsAfterWidgetTracking(NSEvent *initiatingEvent);
+
+    bool lastEventIsMouseUp() const;
+    void setActivationEventNumber(int num) { _activationEventNumber = num; }
+
+    bool eventMayStartDrag(NSEvent *) const;
     void dragSourceMovedTo(const QPoint &loc);
-    void dragSourceEndedAt(const QPoint &loc);
+    void dragSourceEndedAt(const QPoint &loc, NSDragOperation operation);
 
+    bool mayCut();
+    bool mayCopy();
+    bool mayPaste();
     bool tryCut();
     bool tryCopy();
     bool tryPaste();
-
+    
     bool sendContextMenuEvent(NSEvent *);
 
+    // Call this method before handling a new user action, like on a mouse down or key down.
+    // Currently, all this does is clear the "don't submit form twice" data member.
+    void prepareForUserAction();
+
     void clearTimers();
     static void clearTimers(KHTMLView *);
     
@@ -262,15 +301,17 @@ public:
 
     KWQWindowWidget *topLevelWidget();
     
-    QString overrideMediaType();
+    void tokenizerProcessedData();
+
+    QString overrideMediaType() const;
     
     void setMediaType(const QString &);
 
-    void setUsesInactiveTextBackgroundColor(bool u) { _usesInactiveTextBackgroundColor = u; }
-    bool usesInactiveTextBackgroundColor() const { return _usesInactiveTextBackgroundColor; }
+    void setSelectionFromNone();
+    void setDisplaysWithFocusAttributes(bool flag);
+    bool displaysWithFocusAttributes() const;
 
-    void setShowsFirstResponder(bool flag);
-    bool showsFirstResponder() const { return _showsFirstResponder; }
+    void setWindowHasFocus(bool flag);
     
     // Convenience, to avoid repeating the code to dig down to get this.
     QChar backslashAsCurrencySymbol() const;
@@ -284,35 +325,67 @@ public:
     void didTellBridgeAboutLoad(const QString &urlString);
     bool haveToldBridgeAboutLoad(const QString &urlString);
 
-    KJS::Bindings::Instance *getEmbedInstanceForView (NSView *aView);
-    KJS::Bindings::Instance *getAppletInstanceForView (NSView *aView);
+    KJS::Bindings::Instance *getEmbedInstanceForView(NSView *aView);
+    KJS::Bindings::Instance *getObjectInstanceForView(NSView *aView);
+    KJS::Bindings::Instance *getAppletInstanceForView(NSView *aView);
     void addPluginRootObject(const KJS::Bindings::RootObject *root);
     void cleanupPluginRootObjects();
     
-    void registerCommandForUndo(const khtml::EditCommand &);
-    void registerCommandForRedo(const khtml::EditCommand &);
+    void registerCommandForUndo(const khtml::EditCommandPtr &);
+    void registerCommandForRedo(const khtml::EditCommandPtr &);
     void clearUndoRedoOperations();
-    bool interceptEditingKeyEvent();
     void issueUndoCommand();
     void issueRedoCommand();
     void issueCutCommand();
     void issueCopyCommand();
     void issuePasteCommand();
-    void respondToChangedSelection();
+    void issuePasteAndMatchStyleCommand();
+    void respondToChangedSelection(const khtml::Selection &oldSelection, bool closeTyping);
     void respondToChangedContents();
-    bool isContentEditable() const;
-    bool shouldBeginEditing(const DOM::Range &) const;
-    bool shouldEndEditing(const DOM::Range &) const;
+    virtual bool isContentEditable() const;
+    virtual bool shouldBeginEditing(const DOM::Range &) const;
+    virtual bool shouldEndEditing(const DOM::Range &) const;
 
+    KJS::Bindings::RootObject *executionContextForDOM();
     KJS::Bindings::RootObject *bindingRootObject();
     
     WebScriptObject *windowScriptObject();
+    NPObject *KWQKHTMLPart::windowScriptNPObject();
     
     void partClearedInBegin();
     
     // Implementation of CSS property -khtml-user-drag == auto
-    bool shouldDragAutoNode(DOM::NodeImpl*) const;
+    bool shouldDragAutoNode(DOM::NodeImpl*, int x, int y) const;
 
+    struct MarkedTextUnderline {
+        MarkedTextUnderline(unsigned _startOffset, unsigned _endOffset, const QColor &_color, bool _thick) 
+            : startOffset(_startOffset)
+            , endOffset(_endOffset)
+            , color(_color)
+            , thick(_thick)
+        {}
+        unsigned startOffset;
+        unsigned endOffset;
+        QColor color;
+        bool thick;
+    };
+    
+    void setMarkedTextRange(const DOM::Range &, NSArray *attributes, NSArray *ranges);
+    DOM::Range markedTextRange() const;
+    bool markedTextUsesUnderlines() const;
+    QValueList<MarkedTextUnderline> markedTextUnderlines() const;
+
+    bool canGoBackOrForward(int distance) const;
+
+    void didFirstLayout();
+    
+    NSMutableDictionary *dashboardRegionsDictionary();
+    void dashboardRegionsChanged();
+    
+    virtual bool isCharacterSmartReplaceExempt(const QChar &, bool);
+    
+    DOM::NodeImpl *mousePressNode();
+    
 private:
     virtual void khtmlMousePressEvent(khtml::MousePressEvent *);
     virtual void khtmlMouseDoubleClickEvent(khtml::MouseDoubleClickEvent *);
@@ -342,6 +415,12 @@ private:
 
     void freeClipboard();
 
+    khtml::RenderStyle *styleForSelectionStart(DOM::NodeImpl *&nodeToRemove) const;
+
+    virtual bool isKHTMLPart() const;
+
+    void registerCommandForUndoOrRedo(const khtml::EditCommandPtr &cmd, bool isRedo);
+
     WebCoreBridge *_bridge;
     
     KWQSignal _started;
@@ -357,9 +436,10 @@ private:
     int _mouseDownWinX, _mouseDownWinY;
     // in our view's coords
     int _mouseDownX, _mouseDownY;
+    float _mouseDownTimestamp;
+    int _activationEventNumber;
     
     static NSEvent *_currentEvent;
-    static NSResponder *_firstResponderAtMouseDownTime;
 
     KURL _submittedFormURL;
 
@@ -370,8 +450,6 @@ private:
 
     KWQWindowWidget *_windowWidget;
 
-    bool _usesInactiveTextBackgroundColor;
-    bool _showsFirstResponder;
     mutable bool _drawSelectionOnly;
     bool _haveUndoRedoOperations;
     
@@ -383,6 +461,7 @@ private:
                                             // bound outside the context of a plugin.
     QPtrList<KJS::Bindings::RootObject> rootObjects;
     WebScriptObject *_windowScriptObject;
+    NPObject *_windowScriptNPObject;
     
     DOM::Node _dragSrc;     // element that may be a drag source, for the current mouse gesture
     bool _dragSrcIsLink;
@@ -393,6 +472,12 @@ private:
     KWQClipboard *_dragClipboard;   // used on only the source side of dragging
     
     mutable DOM::Node _elementToDraw;
+
+    DOM::Range m_markedTextRange;
+    bool m_markedTextUsesUnderlines;
+    QValueList<MarkedTextUnderline> m_markedTextUnderlines;
+
+    bool m_windowHasFocus;
 };
 
 inline KWQKHTMLPart *KWQ(KHTMLPart *part) { return static_cast<KWQKHTMLPart *>(part); }
