@@ -57,6 +57,7 @@ enum FunctionNumber {
     slotChildCompletedWithBool,
     slotChildStarted,
     slotClicked,
+    slotEndLifeSupport,
     slotFinishedParsing,
     slotLoaderRequestDone,
     slotLoaderRequestStarted,
@@ -82,7 +83,7 @@ enum FunctionNumber {
     slotFinished_KHTMLPart,
     slotFinished_Loader,
     slotFinished_XMLHttpRequest,
-    slotReceivedResponse
+    slotReceivedResponse,
 };
 
 KWQSlot::KWQSlot(QObject *object, const char *member)
@@ -96,6 +97,7 @@ KWQSlot::KWQSlot(QObject *object, const char *member)
     CASE(slotClicked, (), RenderFormElement)
     CASE(slotChildCompleted, (), KHTMLPart)
     CASE(slotChildStarted, (KIO::Job *), KHTMLPart)
+    CASE(slotEndLifeSupport, (), KHTMLPart)
     CASE(slotFinishedParsing, (), KHTMLPart)
     CASE(slotLoaderRequestDone, (khtml::DocLoader *, khtml::CachedObject *), KHTMLPart)
     CASE(slotLoaderRequestStarted, (khtml::DocLoader *, khtml::CachedObject *), KHTMLPart)
@@ -146,16 +148,17 @@ KWQSlot::KWQSlot(QObject *object, const char *member)
 	} else {
 	    m_function = slotRedirection_XMLHttpRequest;
 	}
+    } else if (KWQNamesMatch(member, SLOT(slotFinished(KIO::Job *, NSData *)))) {
+	ASSERT(dynamic_cast<khtml::Loader *>(object));
+	m_function = slotFinished_Loader;        
     } else if (KWQNamesMatch(member, SLOT(slotFinished(KIO::Job *)))) {
-	ASSERT(dynamic_cast<khtml::Loader *>(object) || dynamic_cast<KHTMLPart *>(object) || dynamic_cast<XMLHttpRequestQObject *>(object));
-	if (dynamic_cast<khtml::Loader *>(object)) {
-	    m_function = slotFinished_Loader;
-	} else if (dynamic_cast<KHTMLPart *>(object)) {
+	ASSERT(dynamic_cast<KHTMLPart *>(object) || dynamic_cast<XMLHttpRequestQObject *>(object));
+	if (dynamic_cast<KHTMLPart *>(object)) {
 	    m_function = slotFinished_KHTMLPart;
 	} else {
 	    m_function = slotFinished_XMLHttpRequest;
 	}
-    } else if (KWQNamesMatch(member, SLOT(slotReceivedResponse(KIO::Job *, void *)))) {
+    } else if (KWQNamesMatch(member, SLOT(slotReceivedResponse(KIO::Job *, NSURLResponse *)))) {
 	ASSERT(dynamic_cast<khtml::Loader *>(object));
 	m_function = slotReceivedResponse;
     } else {
@@ -181,6 +184,7 @@ void KWQSlot::call() const
         CASE(signalFinishedParsing, DocumentImpl, m_finishedParsing.call)
         CASE(slotChildCompleted, KHTMLPart, slotChildCompleted)
         CASE(slotClicked, RenderFormElement, slotClicked)
+        CASE(slotEndLifeSupport, KHTMLPart, slotEndLifeSupport)
         CASE(slotFinishedParsing, KHTMLPart, slotFinishedParsing)
         CASE(slotParentCompleted, KHTMLPart, slotParentCompleted)
         CASE(slotParentDestroyed, WindowQObject, parentDestroyed)
@@ -269,9 +273,6 @@ void KWQSlot::call(Job *job) const
         case slotFinished_KHTMLPart:
             static_cast<KHTMLPart *>(m_object.pointer())->slotFinished(job);
             return;
-        case slotFinished_Loader:
-            static_cast<Loader *>(m_object.pointer())->slotFinished(job);
-            return;
         case slotFinished_XMLHttpRequest:
             static_cast<XMLHttpRequestQObject *>(m_object.pointer())->slotFinished(job);
             return;
@@ -316,7 +317,22 @@ void KWQSlot::call(Job *job, const KURL &url) const
     call();
 }
 
-void KWQSlot::call(KIO::Job *job, void *response) const
+void KWQSlot::call(KIO::Job *job, NSData *allData) const
+{
+    if (m_object.isNull()) {
+        return;
+    }
+    
+    switch (m_function) {
+        case slotFinished_Loader:
+            static_cast<Loader *>(m_object.pointer())->slotFinished(job, allData);
+            return;
+    }
+    
+    call(job);
+}
+
+void KWQSlot::call(KIO::Job *job, NSURLResponse *response) const
 {
     if (m_object.isNull()) {
         return;
@@ -327,7 +343,7 @@ void KWQSlot::call(KIO::Job *job, void *response) const
 	    static_cast<Loader *>(m_object.pointer())->slotReceivedResponse(job, response);
 	    return;
     }
-
+    
     call();
 }
 
